export default PatternMatcher;
export type QuadPart = 'subject' | 'predicate' | 'object' | 'graph';
export type TermSet = import('@rdfjs/term-set').default;
/**
 * @typedef {'subject' | 'predicate' | 'object' | 'graph'} QuadPart
 * @typedef {import('@rdfjs/term-set').default} TermSet
 */
declare class PatternMatcher {
    /**
     * @param {object} [options]
     * @param {import('@rdfjs/types').Quad_Subject | Iterable<import('@rdfjs/types').Term>} [options.subject]
     * @param {import('@rdfjs/types').Quad_Predicate | Iterable<import('@rdfjs/types').Term>} [options.predicate]
     * @param {import('@rdfjs/types').Quad_Object | Iterable<import('@rdfjs/types').Term>} [options.object]
     * @param {import('@rdfjs/types').Quad_Graph | Iterable<import('@rdfjs/types').Term>} [options.graph]
     */
    constructor({ subject, predicate, object, graph }?: {
        subject?: import("@rdfjs/types").Quad_Subject | Iterable<import("@rdfjs/types").Term> | undefined;
        predicate?: import("@rdfjs/types").Quad_Predicate | Iterable<import("@rdfjs/types").Term> | undefined;
        object?: import("@rdfjs/types").Quad_Object | Iterable<import("@rdfjs/types").Term> | undefined;
        graph?: import("@rdfjs/types").Quad_Graph | Iterable<import("@rdfjs/types").Term> | undefined;
    } | undefined);
    /**
     * @type {Partial<Record<QuadPart, TermSet>>}
     */
    pattern: Partial<Record<QuadPart, TermSet>>;
    /**
     * @param {import('@rdfjs/types').Quad} quad
     * @returns {boolean}
     */
    test(quad: import('@rdfjs/types').Quad): boolean;
    /**
     * @param {QuadPart} name
     * @param {import('@rdfjs/types').Term | Iterable<import('@rdfjs/types').Term> | undefined} value
     */
    set(name: QuadPart, value: import('@rdfjs/types').Term | Iterable<import('@rdfjs/types').Term> | undefined): void;
}
